/*============================================================================
  File:     IndexUsageStats.sql

  Summary:  This script demonstrates using DMVs to access information about
			index usage stats.
            Check out this blog post for more details: http://www.sqlskills.com/blogs/paul/2007/10/05/IndexesFromEveryAngleHowCanYouTellIfAnIndexIsBeingUsed.aspx
           
  Date:     June 2008

  SQL Server Version: 9.00.3054.00 (SP2)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Look at index usage stats
SELECT * FROM sys.dm_db_index_usage_stats;
GO

-- Now use a filter on the DMV
SELECT * FROM sys.dm_db_index_usage_stats
WHERE database_id = DB_ID('AdventureWorks')
and object_id = OBJECT_ID('AdventureWorks.Person.Address');
GO

-- Do something to use the table
SELECT *
FROM AdventureWorks.Person.Address;
GO

-- Do something to use a non-clustered index
SELECT StateProvinceID
FROM AdventureWorks.Person.Address
WHERE StateProvinceID > 4
AND StateProvinceID < 15;
GO

-- Check "usage stats" again...
SELECT * FROM sys.dm_db_index_usage_stats
WHERE database_id = DB_ID('AdventureWorks')
and object_id = OBJECT_ID('AdventureWorks.Person.Address');
GO

-- Create a table to hold the periodic snapshots of index
-- usage stats
USE PerformanceDW
go

IF OBJECTPROPERTY(object_id(N'MyIndexUsageStats'), 
	'IsUserTable') = 1
	DROP TABLE MyIndexUsageStats;
GO

SELECT GETDATE () AS ExecutionTime, *
INTO PerformanceDW.dbo.MyIndexUsageStats
FROM sys.dm_db_index_usage_stats WHERE database_id=0;
GO

-- Take a baseline snapshot of the index usage stats DMV
INSERT PerformanceDW.dbo.MyIndexUsageStats 
	SELECT getdate (), * 
FROM sys.dm_db_index_usage_stats;
GO

-- Do a bunch of stuff
SELECT * FROM AdventureWorks.Person.Address;
GO
SELECT * FROM AdventureWorks.Person.Address;
GO
SELECT StateProvinceID
FROM AdventureWorks.Person.Address
WHERE StateProvinceID > 4
AND StateProvinceID < 15;
GO

-- Take another snapshot
INSERT PerformanceDW.dbo.MyIndexUsageStats 
	SELECT getdate (), * 
FROM sys.dm_db_index_usage_stats;
GO

-- Look at the history of usage
SELECT * FROM PerformanceDW.dbo.MyIndexUsageStats
WHERE database_id = DB_ID('AdventureWorks')
and object_id = OBJECT_ID('AdventureWorks.Person.Address');
GO
