/*============================================================================
  File:     IndexUsageStats.sql

  Summary:  Be careful... don't just drop all unused indexes! Please use
			this script with caution. If your persisted information is not
			comprehensive then you might be dropping useful indexes - just
			not indexes used since you started persisting the data!!!
           
  Date:     June 2008

  SQL Server Version: 9.00.3054.00 (SP2)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Generically look at all indexes - and persist this data so that you can analyze over time

USE [PerformanceDW]
go

-- drop the table... 
-- drop table [persisted-dm_db_index_usage_stats]

-- Create the permanent table
SELECT getdate() AS RunTime 
	, DB_NAME(i.database_id) as DatabaseName
	, OBJECT_NAME(i.object_id, i.database_id) as ObjectName
	, *
	, 1 AS [Rank]
INTO [persisted-dm_db_index_usage_stats]
FROM sys.dm_db_index_usage_stats AS i
WHERE 1=2 
go

INSERT [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats]
SELECT getdate()  
	, DB_NAME(i.database_id) 
	, OBJECT_NAME(i.object_id, i.database_id) 
	, *
FROM 
	(SELECT *, 
	DENSE_RANK() OVER (PARTITION BY database_id ORDER BY database_id ASC) AS [Rank]
	FROM sys.dm_db_index_usage_stats) AS i
WHERE object_id > 100 AND DB_NAME(i.database_id) NOT IN ('master', 'model', 'msdb', 'tempdb')
go

SELECT *
FROM [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats]
WHERE object_id > 100
ORDER BY DatabaseName ASC
go

-- Verify the indexes NOT used for a particular database
-- Remember, these are the indexes NOT used since you started gathering "usage stats"
-- If you have NOT gathered stats over a true "business cycle" then you may NOT
-- have accurate stats (for example for month end processing) and therefore you 
-- might actually drop an index that's very critical for a report or other and
-- that might have a profound affect on overall performance (during month-end)...

USE credit
go

SELECT object_name(si.object_id) AS 'TableName'
	, si.index_id
	, si.name AS 'IndexName'
	, N'DROP INDEX ' 
		+ QUOTENAME(object_name(si.object_id), N']') 
		+ N'.' 
		+ QUOTENAME(si.name, N']') AS 'Drop Index Statement'
FROM sys.indexes AS si 
WHERE si.index_id 
	NOT IN (SELECT pStats.index_id 
            FROM [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats] AS pStats 
            WHERE pStats.object_id = si.object_id AND 
                  si.index_id = pStats.index_id AND 
                  database_id = db_id()) 
AND si.object_id > 100 
	AND si.index_id > 0 -- don't drop heaps (it would fail regardless because you can't drop a heap with DROP INDEX...)
	AND si.is_primary_key = 0
	AND si.is_unique_constraint = 0
	AND object_name(si.object_id) NOT LIKE 'queue_messages%'
ORDER BY object_name(si.object_id)
go

-----------------------------------------------------------------------
-- Here's a copy of the DMV query if you just want to run it once
-- but include the database and object name instead of just IDs:
-----------------------------------------------------------------------
SELECT getdate() AS RunTime 
	, DB_NAME(i.database_id) as DatabaseName
	, OBJECT_NAME(i.object_id, i.database_id) as ObjectName
	, *
FROM 
	(SELECT *, 
	DENSE_RANK() OVER (PARTITION BY database_id ORDER BY database_id ASC) AS rnk
	FROM sys.dm_db_index_usage_stats) AS i
WHERE object_id > 100
go