SET STATISTICS IO ON 
go

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member 
WHERE lastname LIKE '[S-Z]%'

---------------------------------------
-- What if we had an index on 
-- Lastname ONLY? Would SQL Server use
-- it for such a LOW SELECTIVITY query?
CREATE INDEX NCIndexLNOnly
ON member(lastname)

-- nope... so, let's force it 
-- just to check
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexLNOnly))
WHERE lastname LIKE '[S-Z]%'

-- What does SQL Server do?
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member 
WHERE lastname LIKE '[S-Z]%'

---------------------------------------
-- so, what about covering?
CREATE INDEX NCIndexCoversAll4Cols 
ON member
(lastname, firstname, middleinitial,
	 phone_no)

-- compare this against a table scan
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (0)) --Index 0 forces a table scan
WHERE lastname LIKE '[S-Z]%'

-- What does SQL Server do?
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member 
WHERE lastname LIKE '[S-Z]%'


---------------------------------------
-- how about JUST putting lastname in the key
-- and "including" the other columns
CREATE INDEX NCIndexLNinKeyInclude3OtherCols 
ON member(lastname)
INCLUDE (firstname, middleinitial, phone_no)

-- Compare against the fully covering index
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexCoversAll4Cols))
WHERE lastname LIKE '[S-Z]%'

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexLNinKeyInclude3OtherCols))
WHERE lastname LIKE '[S-Z]%'

-- what about index size?
sp_dbcmptlevel credit, 90
go

select * from sys.dm_db_index_physical_stats
(db_id(), object_id('member'), null, null, 'detailed')
go
-- basically they're the same size!

-- what index would we *really* create??
CREATE INDEX NCIndexCoveringLnFnMiIncludePhone
ON member(lastname, firstname, middleinitial)
INCLUDE (phone_no)

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexCoversAll4Cols))
WHERE lastname LIKE '[S-Z]%'

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexLNinKeyInclude3OtherCols))
WHERE lastname LIKE '[S-Z]%'

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexCoveringLnFnMiIncludePhone))
WHERE lastname LIKE '[S-Z]%'

-- Ok, if they're all the same, then when NCIndexCoveringLnFnMiIncludePhone????
-- imagine an ORDER BY?
SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexCoversAll4Cols))
WHERE lastname LIKE '[S-Z]%'
ORDER BY lastname, firstname, middleinitial

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexLNinKeyInclude3OtherCols))
WHERE lastname LIKE '[S-Z]%'
ORDER BY lastname, firstname, middleinitial

SELECT lastname, firstname, 
	middleinitial, phone_no
FROM member WITH (INDEX (NCIndexCoveringLnFnMiIncludePhone))
WHERE lastname LIKE '[S-Z]%'
ORDER BY lastname, firstname, middleinitial

-- how do these indexes actually differ?

--sp_dbcmptlevel credit, 90
select * from sys.dm_db_index_physical_stats
(db_id(), object_id('member'), null, null, 'detailed')

select * from sys.indexes 
where [object_id] = object_id('member')

-- Can you see INCLUDE columns:
SELECT
(case ic.key_ordinal when 0 then cast(1 as tinyint) else ic.key_ordinal end) AS [ID],
clmns.name AS [Name],
CAST(COLUMNPROPERTY(ic.object_id, clmns.name, N'IsComputed') AS bit) AS [IsComputed],
ic.is_descending_key AS [Descending],
ic.is_included_column AS [IsIncluded]
FROM
sys.tables AS tbl
INNER JOIN sys.indexes AS i ON (i.index_id > 0 and i.is_hypothetical = 0) AND (i.object_id=tbl.object_id)
INNER JOIN sys.index_columns AS ic ON (ic.column_id > 0 and (ic.key_ordinal > 0 or ic.partition_ordinal = 0 or ic.is_included_column != 0)) AND (ic.index_id=CAST(i.index_id AS int) AND ic.object_id=i.object_id)
INNER JOIN sys.columns AS clmns ON clmns.object_id = ic.object_id and clmns.column_id = ic.column_id
WHERE
(i.name=N'test3')and((tbl.name=N'member' and SCHEMA_NAME(tbl.schema_id)=N'dbo'))
ORDER BY IsIncluded, [ID] ASC